
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglStringSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/MimeSym.h"


MimeSym::MimeSym(): RecSym("Mime", new PglRec) {
	theRec->bAdd("string", "type", 0);
	theRec->bAdd("string[]", "prefixes", 0);   // XXX: move to UrlGen
	theRec->bAdd("string[]", "extensions", 0); // XXX: move to UrlGen
}

MimeSym::MimeSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool MimeSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Mime";
}

SynSym *MimeSym::dupe(const String &type) const {
	if (isA(type))
		return new MimeSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

String MimeSym::mimeType() const {
	return getString("type");
}

bool MimeSym::extensions(Array<String*> &exts, RndDistr *&selector) const {
	if (ArraySym *as = getArraySym("extensions")) {
		selector = as->makeSelector("mime-extensions");
		Assert(getStrings("extensions", exts));
		return true;
	}
	return false;
}

bool MimeSym::prefixes(Array<String*> &pxs, RndDistr *&selector) const {
	if (ArraySym *as = getArraySym("prefixes")) {
		selector = as->makeSelector("mime-prefixes");
		Assert(getStrings("prefixes", pxs));
		return true;
	}
	return false;
}
