
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PGL_STRING_RANGE_RANGE_H
#define PGL_STRING_RANGE_RANGE_H

#include "xstd/Array.h"
#include "xparser/LexToken.h"

class ostream;
class String;
class ArraySym;
class PglStrBlock;

// a string range (e.g., "blah[1-100].foo") parser and type converter
class PglStrRange {
	public:
		typedef void (*Iter)(const String &);

	public:
		PglStrRange();
		PglStrRange(const PglStrRange &r);
		virtual ~PglStrRange();

		virtual void reset();

		int count() const;

		void iterate(Iter iter) const;
		String toStr() const;
		void toStrs(Array<String*> &strs) const;
		virtual ArraySym *toSyms(const TokenLoc &loc) const;

		virtual bool parse(const String &val);
		virtual ostream &print(ostream &os) const;

		// used by the lexer only
		void addPointBlock(const char *start, const char *stop);
		void addRangeBlock(int start, int stop, bool isolated);

		bool canMergeStrs(const PglStrRange &r) const;
		virtual void merge(const PglStrRange &r); // aborts if cannot

	protected:
		void startIter() const;
		bool nextIter() const;
		bool nextIter(int level) const;
		void skipIter(int count) const;
		void currentIter(String &str) const;
		int iterPos() const;

		int rangedBlockCnt() const; // number of blocks specified using a range

		virtual bool escapeChar(char ch) const;
		virtual bool optRangeBeg(char ch) const;
		virtual bool reqRangeBeg(char ch) const;
		virtual bool reqRangeEnd(char ch) const;
		virtual bool reqRangeEnd(char beg, char end) const;
		
	protected:
		Array<PglStrBlock*> theBlocks; // range parts
		String theImage;    // original range image
};

#endif
